/* cert.c - LDAP access
 *      Copyright (C) 2002 Klarlvdalens Datakonsult AB
 *
 * This file is part of DirMngr.
 *
 * DirMngr is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DirMngr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <gcrypt.h>
#include "cert.h"

#include "dirmngr.h"

#if 0
void fail_if_err( KsbaError err) 
{
  if(err) {
    dirmngr_debug( "%s:%d: KSBA error: %s\n", 
		   __FILE__, __LINE__, ksba_strerror(err));
    exit (1);
  }
}
#endif

char* hashify_data( const char* data, size_t len )
{
  unsigned char buf[20];
  gcry_md_hash_buffer (GCRY_MD_SHA1, buf, data, len);
  return hexify_data( buf, 20 );
}

char* hexify_data( const unsigned char* data, size_t len )
{
  int i;
  char* result = xmalloc( sizeof( char ) * (2*len+1));

  for( i = 0; i < 2*len; i+=2 )
    sprintf( result+i, "%02X", *data++);
  return result;
}

char* serial_hex( KsbaSexp serial )
{
  unsigned char* p = serial;
  char *endp;
  unsigned long n;
  char *certid;

  if (!p)
    return NULL;
  else {
    p++; /* ignore initial '(' */
    n = strtoul (p, (char**)&endp, 10);
    p = endp;
    if (*p!=':')
      return NULL;
    else {
      int i = 0;
      certid = xmalloc( sizeof( char )*(2*n + 1 ) );
      for (p++; n; n--, p++) {
	sprintf ( certid+i , "%02X", *p);
	i += 2;
      }
    }
  }
  return certid;
}

#if 0
static void
print_sexp (KsbaConstSexp p)
{
  unsigned long n;
  KsbaConstSexp endp;

  if (!p)
    fputs ("none", stderr);
  else
    {
      p++;
      n = strtoul (p, (char**)&endp, 10);
      p = endp;
      if (*p!=':')
        fputs ("ERROR - invalid value", stderr);
      else
        {
          for (p++; n; n--, p++)
            fprintf ( stderr, "%02X", *p);
        }
    }
}

static void
print_dn (char *p)
{

  if (!p)
    fputs ("error", stderr);
  else
    fprintf (stderr, "`%s'", p);
}
#endif


KsbaCert make_cert( char* data, size_t len )
{
  KsbaCert cert;
  KsbaError err;

  cert = ksba_cert_new ();
  if (!cert)
    return NULL;
  err = ksba_cert_init_from_mem( cert, data, len );
  if( err ) return NULL;
  return cert;
}
